@@ -117,6 +117,7 @@ save_pixbuf_handle_success (ScreenshotApplication *self)
 
       save_folder_to_settings (self);
       gtk_widget_destroy (dialog->dialog);
+      g_free (dialog);
     }
   else
     {
@@ -204,6 +205,106 @@ save_pixbuf_ready_cb (GObject *source,
   save_pixbuf_handle_success (self);
 }
 
+static void
+find_out_writable_format_by_extension (gpointer data,
+                                      gpointer user_data)
+{
+  GdkPixbufFormat *format     = (GdkPixbufFormat*) data;
+  gchar          **name       = (gchar **) user_data;
+  gchar          **extensions = gdk_pixbuf_format_get_extensions (format);
+  gchar          **ptr        = extensions;
+  gboolean         found      = FALSE;
+
+  while (*ptr != NULL)
+    {
+      if (g_strcmp0 (*ptr, *name) == 0 &&
+          gdk_pixbuf_format_is_writable (format) == TRUE)
+        {
+          *name = gdk_pixbuf_format_get_name (format);
+	  found = TRUE;
+          break;
+        }
+      ptr++;
+    }
+
+  g_strfreev (extensions);
+
+  /* Needing to duplicate string here because
+   * gdk_pixbuf_format_get_name will return a duplicated string.
+   */
+  if (!found)
+    *name = g_strdup (*name);
+}
+
+static gboolean
+is_png (gchar *format)
+{
+  if (g_strcmp0 (format, "png") == 0)
+    return TRUE;
+  else
+    return FALSE;
+}
+
+static gboolean
+has_profile (ScreenshotApplication *self)
+{
+  if (self->priv->icc_profile_base64 != NULL)
+    return TRUE;
+  else
+    return FALSE;
+}
+
+static void
+save_with_description_and_profile (ScreenshotApplication *self,
+                                   GFileOutputStream     *os,
+                                   gchar                 *format)
+{
+  gdk_pixbuf_save_to_stream_async (self->priv->screenshot,
+                                   G_OUTPUT_STREAM (os),
+                                   format, NULL,
+                                   save_pixbuf_ready_cb, self,
+                                   "icc-profile", self->priv->icc_profile_base64,
+                                   "tEXt::Software", "gnome-screenshot",
+                                   NULL);
+}
+static void
+save_with_description (ScreenshotApplication *self,
+                       GFileOutputStream     *os,
+                       gchar                 *format)
+{
+  gdk_pixbuf_save_to_stream_async (self->priv->screenshot,
+                                   G_OUTPUT_STREAM (os),
+                                   format, NULL,
+                                   save_pixbuf_ready_cb, self,
+                                   "tEXt::Software", "gnome-screenshot",
+                                   NULL);
+}
+
+static void
+save_with_profile (ScreenshotApplication *self,
+                   GFileOutputStream     *os,
+                   gchar                 *format)
+{
+  gdk_pixbuf_save_to_stream_async (self->priv->screenshot,
+                                   G_OUTPUT_STREAM (os),
+                                   format, NULL,
+                                   save_pixbuf_ready_cb, self,
+                                   "icc-profile", self->priv->icc_profile_base64,
+                                   NULL);
+}
+
+static void
+save_with_no_profile_or_description (ScreenshotApplication *self,
+                                     GFileOutputStream     *os,
+                                     gchar                 *format)
+{
+  gdk_pixbuf_save_to_stream_async (self->priv->screenshot,
+                                   G_OUTPUT_STREAM (os),
+                                   format, NULL,
+                                   save_pixbuf_ready_cb, self,
+                                   NULL);
+}
+
 static void
 save_file_create_ready_cb (GObject *source,
                            GAsyncResult *res,
@@ -212,6 +313,24 @@ save_file_create_ready_cb (GObject *source,
   ScreenshotApplication *self = user_data;
   GFileOutputStream *os;
   GError *error = NULL;
+  gchar *basename = g_file_get_basename (G_FILE (source));
+  gchar *extension = g_strrstr (basename, ".");
+  gchar *format = NULL;
+  GSList *formats = NULL;
+
+  if (extension == NULL)
+    extension = "png";
+  else
+    extension++;
+
+  format = extension;
+
+  formats = gdk_pixbuf_get_formats();
+  g_slist_foreach (formats,
+                   find_out_writable_format_by_extension,
+                   (gpointer) &format);
+  g_slist_free (formats);
+  g_free (basename);
 
   if (self->priv->should_overwrite)
     os = g_file_replace_finish (G_FILE (source), res, &error);
@@ -225,23 +344,20 @@ save_file_create_ready_cb (GObject *source,
       return;
     }
 
-  if (self->priv->icc_profile_base64 != NULL)
-    gdk_pixbuf_save_to_stream_async (self->priv->screenshot,
-                                     G_OUTPUT_STREAM (os),
-                                     "png", NULL,
-                                     save_pixbuf_ready_cb, self,
-                                     "icc-profile", self->priv->icc_profile_base64,
-                                     "tEXt::Software", "gnome-screenshot",
-                                     NULL);
+  if (is_png (format))
+    {
+      if (has_profile (self))
+        save_with_description_and_profile (self, os, format);
+      else
+        save_with_description (self, os, format);
+    }
   else
-    gdk_pixbuf_save_to_stream_async (self->priv->screenshot,
-                                     G_OUTPUT_STREAM (os),
-                                     "png", NULL,
-                                     save_pixbuf_ready_cb, self,
-                                     "tEXt::Software", "gnome-screenshot",
-                                     NULL);
+    {
+      save_with_no_profile_or_description (self, os, format);
+    }
 
   g_object_unref (os);
+  g_free (format);
 }
 
 static void
@@ -286,18 +402,12 @@ screenshot_save_to_clipboard (ScreenshotApplication *self)
 }
 
 static void
-screenshot_dialog_response_cb (GtkDialog *d,
-                               gint response_id,
-                               gpointer user_data)
+screenshot_dialog_response_cb (ScreenshotResponse response,
+                               ScreenshotApplication *self)
 {
-  ScreenshotApplication *self = user_data;
-
-  switch (response_id)
+  switch (response)
     {
-    case GTK_RESPONSE_HELP:
-      screenshot_display_help (GTK_WINDOW (d));
-      break;
-    case GTK_RESPONSE_OK:
+    case SCREENSHOT_RESPONSE_SAVE:
       /* update to the new URI */
       g_free (self->priv->save_uri);
       self->priv->save_uri = screenshot_dialog_get_uri (self->priv->dialog);
@@ -307,7 +417,7 @@ screenshot_dialog_response_cb (GtkDialog *d,
       screenshot_save_to_clipboard (self);
       break;
     default:
-      gtk_widget_destroy (GTK_WIDGET (d));
+      g_assert_not_reached ();
       break;
     }
 }
@@ -364,11 +474,10 @@ build_filename_ready_cb (GObject *source,
 
   if (screenshot_config->interactive)
     {
-      self->priv->dialog = screenshot_dialog_new (self->priv->screenshot, self->priv->save_uri);
-      g_signal_connect (self->priv->dialog->dialog,
-                        "response",
-                        G_CALLBACK (screenshot_dialog_response_cb),
-                        self);
+      self->priv->dialog = screenshot_dialog_new (self->priv->screenshot,
+                                                  self->priv->save_uri,
+                                                  (SaveScreenshotCallback)screenshot_dialog_response_cb,
+                                                  self);
     }
   else
     {
@@ -625,40 +734,10 @@ screenshooter_init_stock_icons (void)
   g_object_unref (factory);
 }
 
-static void
-interactive_dialog_response_cb (GtkWidget *d,
-                                gint response,
-                                gpointer user_data)
-{
-  ScreenshotApplication *self = user_data;
-
-  if (response != GTK_RESPONSE_HELP)
-    gtk_widget_destroy (d);
-
-  switch (response)
-    {
-    case GTK_RESPONSE_DELETE_EVENT:
-      break;
-    case GTK_RESPONSE_OK:
-      screenshot_start (self);
-      break;
-    case GTK_RESPONSE_HELP:
-      screenshot_display_help (GTK_WINDOW (d));
-      break;
-    default:
-      g_assert_not_reached ();
-      break;
-    }
-}
-
 static void
 screenshot_show_interactive_dialog (ScreenshotApplication *self)
 {
-  GtkWidget *dialog;
-
-  dialog = screenshot_interactive_dialog_new ();
-  g_signal_connect (dialog, "response",
-                    G_CALLBACK (interactive_dialog_response_cb), self);
+  screenshot_interactive_dialog_new ((CaptureClickedCallback) screenshot_start, self);
 }
 
 static void
